@@ -13,6 +13,7 @@ import os.path
 import traceback
 import numbers
 import string
+import logging
 
 from . import toolbox
 from concurrent import futures
@@ -23,8 +24,6 @@ VERSION = '0.9.8'
 cfg = argparse.Namespace()
 cfg.runtime = argparse.Namespace()
 
-errpipe = toolbox.PipeSet()
-
 
 # Fill default values
 def fill_defaults():
@@ -36,30 +35,21 @@ def fill_defaults():
         cfg.build.compile_cmd = []
 
 
-def info(message):
-    if not cfg.runtime.quiet:
-        sys.stdout.write(message)
-        sys.stdout.flush()
-
-
 # handle command line args
 def parse_arguments():
     global cfg
-    parser = argparse.ArgumentParser(description='A program to run a set of tests for a programming assignment.', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+    parser = argparse.ArgumentParser(description='A program to run a set of tests for a programming assignment.')
     parser.add_help = True
     parser.add_argument('suite_path', nargs='?', default="./", help='path of test suite to load')
-    parser.add_argument('target_path', nargs='?', default="Projects", help='path of the target projects to consider (by subdirectory / folder)')
+    parser.add_argument('target_path', nargs='?', default="Projects", help='path of target projects (by subdirectory)')
     parser.add_argument('-V', '--version', action='version', version='%(prog)s ' + VERSION)
-    parser.add_argument('-q', '--quiet', dest='quiet', action='store_true', help='execute in quiet mode')
-    parser.add_argument('-d', '--debug', dest='debug', action='store_true', help='display debug information')
+    parser.add_argument('-q', '--quiet', dest='INFO', action='store_false', help='execute in quiet mode (console)')
+    parser.add_argument('-w', '--warn', dest='WARN', action='store_true', help='display warning information (console)')
+    parser.add_argument('-d', '--debug', dest='DEBUG', action='store_true', help='capture debug information (logfile)')
     parser.add_argument('-s', '--set', dest='set', default="*", help = 'test only projects designated (e.g., *_LATE*')
     cfg.runtime = parser.parse_args(sys.argv[1:], cfg.runtime)
-
-    # debug program
-    if cfg.runtime.debug:
-        print("SYSTEM ", sys.version)
-
-    return
+    cfg.logformat = "%(message)s"
+#    cfg.logformat = "[%(levelname)s] %(message)s"
 
 
 def build_project(source_root, build_root, build_cfg):
@@ -116,14 +106,14 @@ def run_suite_tests(framework, subject, proj_settings):
         display_name, identifier, points = project
         data_set, score, penalty_totals = run_project_tests(identifier, framework, subject, proj_settings)
 
-        # If the project didn't compile, just print out a single line indicating that.
+        # If the project didn't compile, just add a single line indicating that.
         if isinstance(data_set, str):
-            errpipe.println("Error:", data_set)
+            logging.error("Error: %s" % data_set)
             data_set = [[], ["Grade: 0 (Does not compile / run)"]]
             score = 0
 
         else:
-            # Print out info (for debugging purposes) on the score and penalty values for the project.
+            # Add info on the score and penalty values for the project.
             data_set += [ [], ["Test Cases: %.2f (%.2f%%)" % (score * points, score * 100) ] ]
             overall_penalty = 0
 
@@ -183,7 +173,7 @@ def run_project_tests(name, framework, subject, proj_settings):
         # Add score, run message, and description as applicable
         row.append('%.2f%%' % (case_score * 100))
         row.append(message if message else '')
-        row.append(proj_settings.get_test_description(test_num, context) if case_score < 1 else '')
+        row.append(proj_settings.get_test_description(test_num, context) if round(case_score, 10) < 1 else '')
 
         if case_score == 0:
             data_set.append(row)
@@ -245,24 +235,24 @@ def prepare_and_test_submission(framework_context, submission):
     shutil.copytree(submission, cfg.build.destination, dirs_exist_ok=True)
 
     # Build the project.
-    info("Prepping / building project(s) for " + submission + "... ")
-    result_error = build_project(cfg.build.subject_src, cfg.build.subject_bin, cfg.build)
-    if result_error:
-        info("error building (see logs)... ")
-        errpipe.println("Error prepping/building", submission, "-", type(result_error).__name__ + ":", str(result_error))
+    logging.info("Prepping / building project(s) for " + submission + "... ")
+    error = build_project(cfg.build.subject_src, cfg.build.subject_bin, cfg.build)
+    if error:
+        logging.info("error building (see logs)... ")
+        logging.error("Error prepping/building %s - %s: %s" % (submission, type(error).__name__, error))
 
-    info("Initializing... ")
+    logging.info("Initializing... ")
     subject_context = None
     starting_dir = os.getcwd()
 
     try:
         subject_context = cfg.project.initialize_subject(cfg)
-    except Exception as e:
-        errpipe.print("Error initializing subject", submission, "-", type(e).__name__ + ":", str(e))
-        info("error initializing (see logs)... ")
+    except Exception as error:
+        logging.error("Error initializing subject %s - %s: %s" % (submission, type(error).__name__, error))
+        logging.info("error initializing (see logs)... ")
 
     os.chdir(starting_dir)
-    info("done.\n")
+    logging.info("done.\n")
 
     starting_dir = os.getcwd()
     results = run_suite_tests(framework_context, subject_context, cfg.project)
@@ -274,6 +264,11 @@ def prepare_and_test_submission(framework_context, submission):
 def main():
     global cfg
     parse_arguments()
+    console_logger = toolbox.SelectiveStreamHandler(INFO=cfg.runtime.INFO, WARNING=cfg.runtime.WARN, CRITICAL=True)
+    logging.basicConfig(format=cfg.logformat, level=logging.DEBUG, handlers=[console_logger])
+#    root_logger.addHandler(console_logger)
+    root_logger = logging.getLogger('')
+    console_logger.terminator = ""
 
     # Save the current folder and move to the test suite location.
     starting_dir = os.getcwd()
@@ -300,15 +295,15 @@ def main():
     # Build the environment components (only need to do this once.)
     if cfg.build.framework_src and cfg.build.framework_bin:
         if cfg.build.prep_cmd or cfg.build.compile_cmd:
-            info("Prepping / building framework environment... ")
+            logging.info("Prepping / building framework environment... ")
             result_error = build_project(cfg.build.framework_src, cfg.build.framework_bin, cfg.build)
             if result_error:
-                info(type(result_error).__name__ + ": " + str(result_error) + "\n")
+                logging.info("%s: %s\n" % (type(result_error).__name__, result_error))
                 return
 
-        info("initializing framework... ")
+        logging.info("initializing framework... ")
         framework_context = cfg.project.initialize_framework(cfg)
-        info("done.\n")
+        logging.info("done.\n")
     else:
         framework_context = None
 
@@ -319,13 +314,19 @@ def main():
         if not os.path.isdir(output_dir):
             os.mkdir(output_dir)
 
+        logfile = filename=os.path.join(output_dir, cfg.general.error_log)
+        file_logger = toolbox.SelectiveFileHandler(logfile, mode="w", DEBUG=cfg.runtime.DEBUG, ERROR=True)
+        root_logger.addHandler(file_logger)
+#        logging.basicConfig(filename=logfile, level=cfg.loglevel, filemode="w", force=True)
+
         with futures.ThreadPoolExecutor() as executor:
             future = executor.submit(prepare_and_test_submission, framework_context, submission)
             try:
                 suite_results = future.result()
             except Exception as e:
-                errpipe.println("Error preparing / running", submission, "-", type(e).__name__ + ":", str(e))
-                toolbox.data_to_file(errpipe.read(), os.path.join(output_dir, cfg.general.error_log))
+                logging.error("Error preparing / running %s - %s: %s" % (submission, type(e).__name__, e))
+                root_logger.removeHandler(file_logger)
+                file_logger.close()
                 continue
 
         # Track the scores.
@@ -339,10 +340,10 @@ def main():
                 grand_total += result
 
         file_data.append([ "Overall score: %.2f" % grand_total ])
-        errpipe.println("")
 
         sheet.saveCsv(os.path.join(output_dir, cfg.general.result_file), file_data)
-        toolbox.data_to_file(errpipe.read(), os.path.join(output_dir, cfg.general.error_log))
+        root_logger.removeHandler(file_logger)
+        file_logger.close()
         time.sleep(2)
 
     cfg.project.shutdown_framework(framework_context)
